;**************************************************************************
;                        EduKit84 Example #1: Wire
;**************************************************************************
;
; Task specification
;   Value of the input pin is copied to the output pin, demonstrated by the
;   LED on if the pushbutton is pressed.
;
; Task purpose
;   To learn how to manage microcontroller inputs and outputs.
;
; Task analyse
;   Specify one I/O pin to connect the button (input) and another one for
;   LED (output).
;
; Note
;   Connection is already done on the EduKit84 evaluation board. All you
;   need to do is declare selected I/Os in software.
;
; Read the value on the pin where the button is connected. If the button is
; pressed (log.0), set the LED on (switch the output to the log.0 - see the
; EduKit84 schematics).
;
; Build up the program loop with repeated pushbutton reading and
; conditional turning the LED on.

; DEFINITIONS =============================================================
        list    P=16F84A, N=0, R=hex ;part, no pages in listing, radix
        __config 0x3FF1              ;configuration word (fuses) setup
w       equ     0
f       equ     1
; Special Function Registers ..............................................
status  equ     0x03
porta   equ     0x05
trisa   equ     0x05
portb   equ     0x06
trisb   equ     0x06
; Bit Definitions .........................................................
#define PB      portb,0         ;pushbutton is on the RB0 pin
#define LED     porta,4         ;LED is on the RA4 pin
#define RP0     status,5        ;RP0 is bit 5 in status register

; PROGRAM START & INITIALIZATON ===========================================
        org     0               ;program resides from address  ;0x00
        bsf     RP0             ;bank 1 in RAM memory
        movlw   B'11101111'
        movwf   trisa           ;pin RA4 is an output
        movlw   B'11111111'
        movwf   trisb           ;port RB pins are inputs
        bcf     RP0             ;bank 0 in RAM memory

; MAIN PROGRAM ============================================================
Main    btfss   PB              ;is PB 0 or 1?
        goto    Main_A          ;if PB=0, jump to Main_A
        bsf     LED             ;PB=1, LED off
        goto    Main            ;closes the loop
Main_A  bcf     LED             ;LED on
        goto    Main            ;closes the loop
; =========================================================================
        end