;**************************************************************************
;                         EduKit84 Example #2: Blink
;**************************************************************************
;
; Task specification
;   LED will blink independently. Use indirect addressing for writing into
;   the trisa register.
;
; Task purpose;
;   Using delay loops and indirect addressing.
;
; Task analyse
;   Make a subroutine for the exact delay of a specified time. Turn the LED
;   on, call the delay subroutine, turn the LED off and call the delay
;   subroutine again. Finally, close the program loop by returning to the
;   instruction turning the LED on.

; DEFINITIONS =============================================================
        list    P=16F84A, N=0, R=hex ;part, no pages in listing, radix
        __config 0x3FF1              ;configuration word (fuses) setup
w       equ     0
f       equ     1
; Special Function Registers ..............................................
indf    equ     0x00
status  equ     0x03
fsr     equ     0x04
porta   equ     0x05
trisa   equ     0x85
; User Registers ..........................................................
cnt1    equ     0x0C
cnt2    equ     0x0D
cnt3    equ     0x0E
; Bit Definitions .........................................................
#define         LED     porta,4 ;LED on the RA4 pin

; PROGRAM START ===========================================================
        org     0
        goto    Main            ;jump to beginning of program

; SUBROUTINES =============================================================
Wait    movlw   D'250'
        movwf   cnt3            ;load of 3rd loop register
Wait_A  movlw   D'100'
        movwf   cnt2            ;load of 2nd loop register
Wait_B  movlw   D'10'
        movwf   cnt1            ;load of 1st loop register
Wait_C  decfsz  cnt1,f          ;decrement of 1st loop register
        goto    Wait_C          ;jump if register=0
        decfsz  cnt2,f          ;decrement of 2nd loop register
        goto    Wait_B          ;jump if register=0
        decfsz  cnt3,f          ;decrement of 3rd loop register
        goto    Wait_A          ;jump if register=0
        return                  ;return to the main program

; MAIN PROGRAM ============================================================
Main    movlw   trisa
        movwf   fsr             ;to indirect address of trisa
        movlw   B'11101111'
        movwf   indf            ;write to trisa
Main_A  bcf     LED             ;LED on
        call    Wait            ;call delay subroutine
        bsf     LED             ;LED off
        call    Wait            ;call delay subroutine
        goto    Main_A          ;jump to loop beginning
; =========================================================================
        end