;**************************************************************************
;                       EduKit84 Example #4: Display
;**************************************************************************
;
; Task specification
;   If one of eight push buttons is pressed, the display shows 1 - 8.
;
; Task purpose
;   To learn basic manipulation with seven-segment display.
;
; Task analysis
;   The first step is to create a subroutine with a parameter in the W
;   register (1 - 8) which returns (also in W) corresponding value for
;   driving the display (the combination of segments for desired digit).
;   Main program tests pushbuttons (port B pins are inputs at that moment).
;   If any button is pressed, its number (1 - 8) is written to W and the
;   subroutine is called. The result is sent to port B (port B pins are
;   outputs now). Turn the seven-segment display on by clearing the
;   corresponding bit in register A. Then the delay subroutine is called
;   (about 0.4s - almost arbitrary value). Last instruction provides a jump
;   to the beginning of the main loop of the program.

; DEFINITIONS =============================================================
        list    P=16F84A, N=0, R=hex ;part, no pages in listing, radix
        __config 0x3FF1              ;configuration word (fuses) setup
w       equ     0
f       equ     1
; Special Function Registers ..............................................
tmr0    equ     0x01
optreg  equ     0x01
pcl     equ     0x02
status  equ     0x03
porta   equ     0x05
trisa   equ     0x05
portb   equ     0x06
trisb   equ     0x06
intcon  equ     0x0B
; User Registers ..........................................................
disp    equ     0x0C            ;character memory
; Bit Definitions .........................................................
#define RP0     status,5
#define T0IF    intcon,2        ;TMR0 overflow indication

; PROGRAM START ===========================================================
        org     0
        goto    Main

; SUBROUTINES =============================================================
Wait    bcf     T0IF            ;T0IF flag clear
        clrf    tmr0            ;TMR0 and the prescaler clear
Wait_A  btfss   T0IF            ;TMR0 overflow test
        goto    Wait_A          ;test until if TMR0 overflow
        bcf     T0IF            ;T0IF flag clear
        return
; .........................................................................
Table   addwf   pcl,f           ;adds W to PC
        retlw   b'11000000'     ;displays 0
        retlw   b'11111001'     ;displays 1
        retlw   b'10100100'     ;displays 2
        retlw   b'10110000'     ;displays 3
        retlw   b'10011001'     ;displays 4
        retlw   b'10010010'     ;displays 5
        retlw   b'10000010'     ;displays 6
        retlw   b'11111000'     ;displays 7
        retlw   b'10000000'     ;displays 8
        retlw   b'10010000'     ;displays 9

; MAIN PROGRAM ============================================================
Main    bsf     RP0             ;RAM bank 1
        movlw   B'11010111'
        movwf   optreg          ;TMR0 and prescaler configuration
        bcf     RP0             ;RAM bank 0
; .........................................................................
Main_A  bsf     RP0             ;RAM bank 1
        movlw   0xFF
        movwf   portb           ;port B is input
        movlw   0xFF            ;port A is input
        movwf   porta
        bcf     RP0             ;RAM bank 0
        movlw   0x01
        call    Table           ;character 1 code
        btfss   portb,0
        goto    Main_B          ;if pushbutton pressed then display 1
        movlw   0x02
        call    Table           ;character 2 code
        btfss   portb,1
        goto    Main_B          ;if pushbutton pressed then display 2
        movlw   0x03
        call    Table           ;character 3 code
        btfss   portb,2
        goto    Main_B          ;if pushbutton pressed then display 3
        movlw   0x04
        call    Table           ;character 4 code
        btfss   portb,3
        goto    Main_B          ;if pushbutton pressed then display 4
        movlw   0x05
        call    Table           ;character 5 code
        btfss   portb,4
        goto    Main_B          ;if pushbutton pressed then display 5
        movlw   0x06
        call    Table           ;character 6 code
        btfss   portb,5
        goto    Main_B          ;if pushbutton pressed then display 6
        movlw   0x07
        call    Table           ;character 7 code
        btfss   portb,6
        goto    Main_B          ;if pushbutton pressed then display 7
        movlw   0x08
        call    Table           ;character 8 code
        btfss   portb,7
        goto    Main_B          ;if pushbutton pressed then display 8
        goto    Main_A          ;if no pushbutton pressed
                                ;  then jump to beginning
; .........................................................................
Main_B  movwf   portb           ;write a character to portb
        bsf     RP0             ;RAM bank 1
        movlw   0x00
        movwf   portb           ;port B is output
        movlw   B'11111110'     ;activates first 7-segment ;display
        movwf   porta
        bcf     RP0             ;RAM bank 0
        bcf     porta,0         ;first 7-segment display On
        call    Wait
        goto    Main_A          ;jump to beginning
; =========================================================================
        end