; =========================================================================
;                 CLOCK: EduKit84 Demonstration Program
; =========================================================================
; Device: PIC16F84A-04/P, Osc: XT, WDT: Off, Put: On, CP: Off.
; 24-hour clock. Multiplexed LED display demonstration.
; =========================================================================
#include "Def.INC"      ; include source file with definitions
; .........................................................................
        org     0       ; PIC16F84A reset vector
        goto    Start   ; jump to program beginning
; .........................................................................
#include "Int.INC"      ; include file with Interrupt Service Routine
; .........................................................................
#include "Sub.INC"      ; include file with subroutines
; .........................................................................
Start   call    Init            ; initialization routine

Real    movlw   D'200'
        movwf   timer           ; set main timer timeout to 1 second
Real_a  decfsz  timer, f        ; 1 sec is over?
        goto    real_b          ; not yet  => continue
        movlw   .200
        movwf   timer           ; yes => set timer again
        btfsc   stat1, stop     ; clock running? (bit stop=0?)
        goto    real_b          ; no  => continue
        movlw   sec             ; yes => update the time
        movwf   fsr
        movlw   60h
        movwf   maxim           ; selection of seconds
        call    update          ; update seconds
        btfss   status, zero      ; sec = 60? (Z=1?)
        goto    real_b          ; no  => continue
        movlw   min
        movwf   fsr             ; yes => selection of minutes
        call    update          ; update minutes
        btfss   status, zero      ; min = 60? (Z=1?)
        goto    real_b          ; no  => continue
        movlw   hrs
        movwf   fsr
        movlw   24h
        movwf   maxim           ; yes => selection of hours
        call    update          ; update hours
real_b  call    button          ; calling a routine for buttons
        btfsc   stat1, test     ; test mode? (bit test=1?)
        goto    testr           ; yes => test mode
        call    disply          ; time display
        call    w5ms            ; wait 5ms
        goto    real_a          ; and again
;***************************************************************************
; This routine tests the condition of buttons 6 and 7. Every pushing of the
; 6 button sets one of the segments on that's why all segments of all displays
; as well as all LEDs will light on step by step. The test mode is terminated
; by button 7 and the time is displayed again.
; The clock behaviour in the test mode is the same as without testing, i.e.
; it depends on the state of the stop bit. Jumper P1 must be connected CO-AN5.
; For sequentional lighting registers segmt and anode are used. The segmt
; register is sent to port B and zero rotates in it after pushing the key or
; due to autorepeat. That is why LED segments are set on step by step. Thus,
; the segmt register provides driving of cathodes. Driving of anodes is
; controlled via the anode register. It contains rotating values
; 1111, 0111, 1011, 1101, 1110 and so on.
; Rotation is always invoked after passing zero through the whole segmt
; register. The anode register is sent to the potr A and in case of 1111
; the C0 pin activates. Thus, it activates one of the common anode of
; either one of the displays or eight LEDs D1-D8. To avoid switching of
; values if no button is pushed and segmt = 0ffh, there is a bit chang
; which enables changes. After enabling, this bit is cleared and will be
; set again when segmt <> 0ffh.

; Note: because of port C is not implemented for PIC16F84A, the only way
; to test LEDs is set the jumper P1 to AN3-AN5. Then LEDs will be on
; as well as the DISP3.

testr   movlw   b'11111111'
        movwf   porta           ; disconnect display anodes
        movwf   portb           ; disconnect display segments and cathodes D1 to D8
        movwf   segmt           ; code to the segmt register
        movlw   b'11101111'
        movwf   anode           ; initial value to the anode register
        bcf     stat1, push     ; button released, bit cleared
        bcf     stat1, ah       ; disable autorepeat of hours
        bcf     stat1, am       ; disable autorepeat of minutes
        bsf     stat1, chang    ; enable changes
        clrf    cnt_a           ; autorepeat counter cleared
test_a  decfsz  timer, f        ; 1s passed?
        goto    test_b          ; no  => continue
        movlw   .200
        movwf   timer           ; ano => set the counter again
        btfsc   stat1, stop     ; the clock is running? (bit stop=0?)
        goto    test_b          ; no  => continue
        movlw   sec             ; yes => update time
        movwf   fsr
        movlw   60h
        movwf   maxim           ; selection of seconds
        call    update          ; update seconds
        btfss   status, zero      ; sec = 60? (Z=1?)
        goto    test_b          ; no  => continue
        movlw   min
        movwf   fsr             ; yes => selection of minutes
        call    update          ; update minutes
        btfss   status, zero      ; min = 60? (Z=1?)
        goto    test_b          ; no  => continue
        movlw   hrs
        movwf   fsr
        movlw   24h
        movwf   maxim           ; yes => selection of hours
        call    update          ; update hours
test_b  call    pbtest          ; routine for pushbuttons
        btfss   stat1, test     ; test mode? (bit test=1?)
        goto    real_a          ; no  => real time without change of the timer register
        bsf     porta, 4        ; disconnection of the anodes D1 to D8
        movlw   b'11111111'
        movwf   porta           ; disconnection of display anodes
        movf    segmt, w
        movwf   portb           ; the code of diodes or segments to port B
        movlw   0ffh
        xorwf   segmt, w
        btfss   status, zero    ; segmt = 0ffh? (Z=1?)
        goto    test_c          ; no  => use the same value
                                ; yes => generation of next register value
                                ;        anode depending on the bit chang
        btfss   stat1, chang    ; generation allowed? (bit chang = 1?)
        goto    test_c          ; no  => use the same value
                                ; yes => generate next value
        bcf     stat1, chang    ; changes disabled
        bsf     status, carry   ; set carry for rotation
        rrf     anode, f        ; next value
        movf    anode, w
        xorlw   b'11111111'
        btfss   status, zero    ; last value? (Z=1?)
        goto    test_c          ; no  => nothing happens
        movlw   b'11101111'     ; yes => force first value
        movwf   anode
test_c  movf    anode, w
        movwf   porta           ; activation of the common anode of one display
        incf    cnt_m, f        ; increment cnt_m
        movlw   b'00000011'
        andwf   cnt_m, f        ; mask. Now cnt_m counts up to 3 only
        call    w5ms            ; wait 5ms
        goto    test_a          ; and again
; .........................................................................
#include "EEPROM.INC"   ; include file for EEPROM contents definition
; .........................................................................
        end             ; end of program, obligatory directive
